function pmces_binary(ctl,config_options)
%function pmces_binary(ctl,config_options)
%
% Reference: "Co-channel interference separation"
% Pattern analysis and recognition corporation
% Dr. Robert J. Dick
% RADC-TR-80-365 Final Technical Report, December 1980
%
% Rome Air Development Center
% Air Force Systems Command
% Griffis Air Force Base, NY 13441
%
% Adapted and implemented by Mark Harvilla
% Carnegie Mellon University
% June 16, 2012

%%%%%%%%%%%%
% default parameters
% and reading config file
if nargin == 1
	shiftFile = 'none';
	minShift  = 0;
	maxShift  = 300;
	fs        = 16000; 
	T_wind    = 0.08;
	T_shift   = 0.04;
	minFreq   = 55;
	maxFreq   = 300;
	freqRes   = 5;
else
	fid_param = fopen(config_options);
	CO1 = textscan(fid_param,'%s%s','MultipleDelimsAsOne',1);
	fclose(fid_param);

	pVals = CO1{2};

	shiftFile = pVals{1};
	minShift  = str2double(pVals{2});
	maxShift  = str2double(pVals{3});
	fs        = str2double(pVals{4});
	T_wind    = str2double(pVals{5});
	T_shift   = str2double(pVals{6});
	minFreq   = str2double(pVals{7});
	maxFreq   = str2double(pVals{8});
	freqRes   = str2double(pVals{9});
end

%T_wind   = 0.08;
%T_shift  = T_wind/2;
%fs       = 16000;
iFFTSize = floor(T_wind*fs);

%minFreq  = 55;
%maxFreq  = 300;
minPtVal = floor(fs/maxFreq);
maxPtVal = floor(fs/minFreq);

%freqRes  = 5;
histVals = -(maxShift-freqRes):freqRes:maxShift;

if strcmp(shiftFile,'none') ~= 1
	fid_shift = fopen(shiftFile,'w'); %open file to write out shift values
end

fid = fopen(ctl);
L1  = textscan(fid,'%s%s','delimiter',',');
fclose(fid);
inList = L1{1}; outList = L1{2}; shifts = zeros(length(inList),1);
for cnt = 1:length(inList)
 tline = strtrim(inList{cnt});
 fid2  = fopen(tline,'r');

 outFile  = strtrim(outList{cnt});
 file_dir = fileparts(outFile);
 if ~exist(file_dir,'dir')
	mkdir(file_dir);
 end 

 if strcmp(tline(end-2:end),'wav') == 1 % <-- MS-WAV format
     fseek(fid2, 44, 'bof');
 else % <-- else assumed to be SPHERE format
     fseek(fid2, 1024, 'bof');
 end
 x_in = fread(fid2, 'int16');
 x_in = x_in/32768;
 fclose(fid2); 
 
 X = extract_spec(x_in,T_wind,T_shift,fs,iFFTSize);

 iNumWindows = size(X,2);
 cCorr       = zeros(iFFTSize,iNumWindows);
 %cCorr       = zeros(iFFTSize/2 + 1,iNumWindows);
 F0          = zeros(iNumWindows,1);
 df          = zeros(iNumWindows,1);
 fundShift   = zeros(iNumWindows,1);
 %mV1         = zeros(iNumWindows,1);
 %nums        = zeros(1,10000);

 manHist = zeros(1,length(histVals));
 for k = 1:iNumWindows
     X_frame    = X(:,k);
     X_frame    = [X_frame;zeros(iFFTSize-size(X,1),1)];
     cCorr(:,k) = ifft(X_frame);
     corrMag    = abs(cCorr(:,k));
     corrMagSqr = corrMag.^2;
     corrRe     = real(cCorr(:,k));
     corrIm     = imag(cCorr(:,k));
     [mV,mInd]  = max(corrMagSqr(minPtVal:maxPtVal));
    
     mInd = mInd + minPtVal - 1;
    
     if mInd == minPtVal || mInd == maxPtVal %if endpoints, don't interpolate
       F0(k) = fs/mInd;
     
       C_Tb_I = corrIm(mInd);
       C_Tb_R = corrRe(mInd);
     else %if not endpoints, interpolate
       mV1    = corrMagSqr(mInd-1); 
       mV2    = corrMagSqr(mInd+1);
       C1     = 0.5*(mV2 - mV1); 
       C2     = 0.5*(mV1 + mV2) - mV;
       avgInd = mInd - C1/(2*C2);
       F0(k)  = fs/avgInd;

       R1     = 0.5*(corrRe(mInd+1) - corrRe(mInd-1));
       R2     = 0.5*(corrRe(mInd-1) + corrRe(mInd+1)) - corrRe(mInd);
       I1     = 0.5*(corrIm(mInd+1) - corrIm(mInd-1));
       I2     = 0.5*(corrIm(mInd-1) + corrIm(mInd+1)) - corrIm(mInd);
       C_Tb_R = corrRe(mInd) + (avgInd - mInd)*(R1 + (avgInd - mInd)*R2);
       C_Tb_I = corrIm(mInd) + (avgInd - mInd)*(I1 + (avgInd - mInd)*I2);
     end
     d     = atan2(-C_Tb_I,C_Tb_R);
     df(k) = d*F0(k)/(2*pi);
    
     fundShift(k) = df(k);
     nums1        = fundShift(k):F0(k):maxShift;
     nums2        = fliplr((fundShift(k)-F0(k)):-F0(k):-maxShift);
     nums         = [nums2,nums1];
    
     indx            = quantiz(nums,-(maxShift-freqRes):freqRes:maxShift-freqRes);
     manHist(indx+1) = manHist(indx+1) + mV;
 %     A = (k == 1) || (k == iNumWindows);
 %     if mod(k,50) == 0 || A
 %      figure,bar(histVals,manHist);
 %     end
 end
 [~,HmaxInd] = max(manHist);
 fshift      = histVals(HmaxInd); shifts(cnt) = -fshift;
 
 if abs(fshift) > minShift
 	y = freqshift(x_in,fs,fshift);
 else
	y = x_in;
 end
 
 if strcmp(shiftFile,'none') ~= 1
 	fprintf(fid_shift,'%f\n',shifts(cnt));  
 end
 
 wavwrite(0.9*y/max(abs(y)),fs,outFile);
 
 %if mod(cnt,100) == 0
 %   fprintf('%d files processed\n',cnt);
 %end
end
if strcmp(shiftFile,'none') ~= 1
	fclose(fid_shift);
end
